@@ -12,17 +12,25 @@ module Util where
 
 -- Pretty-printing utilities
 
+str :: String -> String -> String
 str = showString
+char :: Char -> String -> String
 char c = (c :)
 
+nl :: String -> String
 nl = char '\n'
 
+paren :: (String -> String) -> String -> String
 paren s = char '(' . s . char ')'
+
+brack :: (String -> String) -> String -> String
 brack s = char '[' . s . char ']'
 
-interleave_shows s [] = id
+interleave_shows :: (String -> String) -> [String -> String] -> String -> String
+interleave_shows _ [] = id
 interleave_shows s xs = foldr1 (\a b -> a . s . b) xs
 
+space :: String -> String
 space = char ' '
 
 cjustify, ljustify, rjustify :: Int -> String -> String
@@ -35,4 +43,5 @@ rjustify n s = spaces (n - length s) ++ s
 spaces   :: Int -> String
 spaces n = replicate n ' '
 
+hline :: String
 hline = replicate 77 '-'
